import { Dispatch } from "./dispatch/Dispatch";
import { ILogItem } from "./ILogItem";
import { LogLevel } from "./LogLevel";

let CURRENT_DISPATCH: Dispatch = new class extends Dispatch {
    output(item: ILogItem): void {
        process.stdout.write(`level{${item.level}} timestamp{${item.timestamp}} origin{${item.origin}} flags{${item.flags}} data{${item.data}}\n`)
    }
};

export const setDispatch = (dispatch: Dispatch) => CURRENT_DISPATCH = dispatch;

export const LOGT = (...args: any[]) => CURRENT_DISPATCH.process({ timestamp: new Date(), level: LogLevel.Trace, data: args });
export const LOGD = (...args: any[]) => CURRENT_DISPATCH.process({ timestamp: new Date(), level: LogLevel.Debug, data: args });
export const LOGI = (...args: any[]) => CURRENT_DISPATCH.process({ timestamp: new Date(), level: LogLevel.Info, data: args });
export const LOGW = (...args: any[]) => CURRENT_DISPATCH.process({ timestamp: new Date(), level: LogLevel.Warn, data: args });
export const LOGE = (...args: any[]) => CURRENT_DISPATCH.process({ timestamp: new Date(), level: LogLevel.Error, data: args });
export const LOGC = (...args: any[]) => CURRENT_DISPATCH.process({ timestamp: new Date(), level: LogLevel.Critical, data: args });

export const LOG = (item: Partial<ILogItem>) => CURRENT_DISPATCH.process({
    flags: item.flags ?? [],
    origin: item.origin ?? "unknown",
    level: item.level ?? LogLevel.Trace,
    timestamp: item.timestamp ?? new Date(),
    data: item.data ?? ""
})